@@ -1,12 +1,12 @@
+import aws_cdk as cdk
+from constructs import Construct
 from aws_cdk import (
-    core,
     aws_rds as _rds,
     aws_ec2 as _ec2,
-    aws_s3 as _s3,
-    aws_secretsmanager as _secretsmanager,
     aws_kms as _kms,
+    # aws_route53 as route53,
+    # aws_route53_targets as route53_targets,
 )
-from aws_cdk.core import Tags
 from aws_cdk_constructs.utils import normalize_environment_parameter, get_version
 
 engine_to_cdk_engine = {
@@ -39,7 +39,7 @@ engine_to_version_class = {
     "postgresql": _rds.PostgresEngineVersion
 }
 
-class Database(core.Construct):
+class Database(Construct):
     """
     A CDK construct to create a "database tier" for your system.
     The construct will make easy to develop a fully compliant database component that includes RDS cluster or instance.
@@ -144,7 +144,7 @@ class Database(core.Construct):
 
     def __init__(
         self,
-        scope: core.Construct,
+        scope: Construct,
         id:str,
         app_name,
         environment,
@@ -173,15 +173,16 @@ class Database(core.Construct):
     ):
         super().__init__(scope, id, **kwargs)
         environment = normalize_environment_parameter(environment)
+        app_name = app_name.lower().strip()
 
         # Apply mandatory tags
-        Tags.of(self).add("ApplicationName", app_name.lower().strip())
-        Tags.of(self).add("Environment", environment)
+        cdk.Tags.of(self).add("ApplicationName", app_name)
+        cdk.Tags.of(self).add("Environment", environment)
 
         # Apply FAO CDK tags
-        Tags.of(self).add("fao-cdk-construct", "database")
-        Tags.of(core.Stack.of(self)).add("fao-cdk-version", get_version())
-        Tags.of(core.Stack.of(self)).add("fao-cdk", "true")
+        cdk.Tags.of(self).add("fao-cdk-construct", "database")
+        cdk.Tags.of(cdk.Stack.of(self)).add("fao-cdk-version", get_version())
+        cdk.Tags.of(cdk.Stack.of(self)).add("fao-cdk", "true")
 
         # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Create conditions
 
@@ -191,9 +192,9 @@ class Database(core.Construct):
         vpc = _ec2.Vpc.from_vpc_attributes (self, 
             "VPC", 
             vpc_id=aws_account["vpc"], 
-            public_subnet_ids=["subnet-8ccc3cea", "subnet-1b3ff853"], 
-            private_subnet_ids=["subnet-1bc5357d", "subnet-db37f093"],
-            availability_zones=["eu-west-1a", "eu-west-1b"]
+            public_subnet_ids=aws_account["public_subnet_ids"], 
+            private_subnet_ids=aws_account["private_subnet_ids"], 
+            availability_zones=aws_account["availability_zones"], 
         )
 
         is_production = environment == "production"
@@ -230,6 +231,21 @@ class Database(core.Construct):
             and rubrik_backup.lower() == "true"
         )
 
+        output_id = (app_name + "-" + environment + "-")
+        identifier_prefix = output_id.replace("_", "-")
+
+
+        # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ CloudFormation outputs
+        cdk.CfnOutput(self, f"DatabaseAppName{app_name}{output_id}", value=str(app_name))
+        cdk.CfnOutput(self, f"DatabaseIsHa{app_name}{output_id}", value=str(is_ha))
+        cdk.CfnOutput(self, f"DatabaseEngine{app_name}{output_id}", value=str(database_engine))
+        cdk.CfnOutput(self, f"DatabaseEngineVersion{app_name}{output_id}", value=str(database_engine_version))
+        # cdk.CfnOutput(self, f"DatabaseAllocatedStorage{app_name}{output_id}", value=str(database_allocated_storage))
+        cdk.CfnOutput(self, f"DatabaseWillSendEmail{app_name}{output_id}", value=str(sends_emails))
+        cdk.CfnOutput(self, f"DatabaseHasBackup{app_name}{output_id}", value=str(to_be_backed_up))
+        cdk.CfnOutput(self, f"DatabaseName{app_name}{output_id}", value=str(database_name))
+        cdk.CfnOutput(self, f"DatabaseIdentifierPrefix{app_name}{output_id}", value=str(identifier_prefix))
+
         # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Validate input params
 
         # TODO
@@ -288,7 +304,6 @@ class Database(core.Construct):
         # AWS Secret Manager
         # ~~~~~~~~~~~~~~~~
         credentials = _rds.Credentials.from_username(database_master_username)
-        identifier_prefix = (app_name + "-" + environment + "-").replace("_", "-")
 
         # ~~~~~~~~~~~~~~~~
         # KMS Encryption key
@@ -337,7 +352,7 @@ class Database(core.Construct):
                 # No need to create instance resource, only specify the amount
                 instances=2 if is_ha else 1,
                 backup=_rds.BackupProps(
-                    retention=core.Duration.days(30), preferred_window="01:00-02:00"
+                    retention= cdk.Duration.days(30), preferred_window="01:00-02:00"
                 ),
                 default_database_name="fao_default_schema",
                 preferred_maintenance_window="mon:03:00-mon:04:00",
@@ -347,6 +362,22 @@ class Database(core.Construct):
                 s3_import_buckets=s3_import_buckets
             )
 
+            self._cluster.add_rotation_single_user(
+                automatically_after= cdk.Duration.days(30)
+            )
+            
+            # Create DNS record
+            # hosted_zone_id = aws_account["route53_hosted_zone_id"]
+            # domain_name = aws_account["route53_domain_name"]
+            # dns_record = app_name if app_name else dns_record
+            # route53_zone = route53.PrivateHostedZone.from_hosted_zone_attributes(self, f"RDSPrivateHostedZone{dns_record}", hosted_zone_id=hosted_zone_id, zone_name=domain_name)
+            # route53.ARecord(self, 
+            #     f"RDSAliasRecord{dns_record}",
+            #     zone=route53_zone,
+            #     target=cdk.Token.as_string(self._cluster.cluster_endpoint), # << Not working here
+            #     record_name=f"{app_name}.{domain_name}"
+            # )
+
             # Conditionally create a cluster from a snapshot
             if use_snapshot:
                 self._cluster.node.find_child("Resource").add_property_override(
@@ -363,16 +394,16 @@ class Database(core.Construct):
                 instance = self._cluster.node.find_child("Instance1")
 
                 if tag_scheduler_uptime_skip:
-                    Tags.of(clus).add("SchedulerSkip", tag_scheduler_uptime_skip)
-                    Tags.of(instance).add("SchedulerSkip", tag_scheduler_uptime_skip)
+                    cdk.Tags.of(clus).add("SchedulerSkip", tag_scheduler_uptime_skip)
+                    cdk.Tags.of(instance).add("SchedulerSkip", tag_scheduler_uptime_skip)
 
                 if tag_scheduler_uptime:
-                    Tags.of(clus).add("SchedulerUptime", tag_scheduler_uptime)
-                    Tags.of(instance).add("SchedulerUptime", tag_scheduler_uptime)
+                    cdk.Tags.of(clus).add("SchedulerUptime", tag_scheduler_uptime)
+                    cdk.Tags.of(instance).add("SchedulerUptime", tag_scheduler_uptime)
 
                 if tag_scheduler_uptime_days:
-                    Tags.of(clus).add("SchedulerUptimeDays", tag_scheduler_uptime_days)    
-                    Tags.of(instance).add("SchedulerUptimeDays", tag_scheduler_uptime_days)    
+                    cdk.Tags.of(clus).add("SchedulerUptimeDays", tag_scheduler_uptime_days)    
+                    cdk.Tags.of(instance).add("SchedulerUptimeDays", tag_scheduler_uptime_days)    
 
         # ~~~~~~~~~~~~~~~~
         # RDS Instance
@@ -430,7 +461,7 @@ class Database(core.Construct):
                 instance_type=instance_type,
                 vpc=vpc,
                 auto_minor_version_upgrade=True,
-                backup_retention=core.Duration.days(30),
+                backup_retention= cdk.Duration.days(30),
                 copy_tags_to_snapshot=True,
                 deletion_protection=is_production,
                 instance_identifier=identifier_prefix + "db",
@@ -456,24 +487,24 @@ class Database(core.Construct):
                 )
 
             self._instance.add_rotation_single_user(
-                automatically_after=core.Duration.days(30)
+                automatically_after= cdk.Duration.days(30)
             )
 
             # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ FAO AWS Rubrik backup tag
 
             if to_be_backed_up:
-                Tags.of(self._instance).add("RubrikBackup", "true")
+                cdk.Tags.of(self._instance).add("RubrikBackup", "true")
             else:
-                Tags.of(self._instance).add("RubrikBackup", "false")
+                cdk.Tags.of(self._instance).add("RubrikBackup", "false")
 
             # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ FAO AWS Scheduler tags
             if is_not_production:
                 instance = self._instance.node.find_child("Resource")
                 if tag_scheduler_uptime_skip:
-                    Tags.of(instance).add("SchedulerSkip", tag_scheduler_uptime_skip)
+                    cdk.Tags.of(instance).add("SchedulerSkip", tag_scheduler_uptime_skip)
 
                 if tag_scheduler_uptime:
-                    Tags.of(instance).add("SchedulerUptime", tag_scheduler_uptime)
+                    cdk.Tags.of(instance).add("SchedulerUptime", tag_scheduler_uptime)
 
                 if tag_scheduler_uptime_days:
-                    Tags.of(instance).add("SchedulerUptimeDays", tag_scheduler_uptime_days) 
+                    cdk.Tags.of(instance).add("SchedulerUptimeDays", tag_scheduler_uptime_days) 
