@@ -1,8 +1,10 @@
-from aws_cdk import core, aws_iam as _iam
-from aws_cdk.core import Tags
+import aws_cdk as cdk
+from constructs import Construct
+from aws_cdk import aws_iam as _iam
+
 from aws_cdk_constructs.utils import normalize_environment_parameter, get_version
 
-class ServiceUserForIAC(core.Construct):
+class ServiceUserForIAC(Construct):
     """Construct to create the Service User to deploy a stack from a CD/CI pipeline
 
     Args:
@@ -30,7 +32,7 @@ class ServiceUserForIAC(core.Construct):
 
     def __init__(
         self,
-        scope: core.Construct,
+        scope: Construct,
         id: str,
         app_name,
         environment,
@@ -41,15 +43,16 @@ class ServiceUserForIAC(core.Construct):
     ) -> None:
         super().__init__(scope, id, **kwargs)
         environment = normalize_environment_parameter(environment)
+        is_production = environment == "Production"
 
         # Apply mandatory tags
-        Tags.of(self).add("ApplicationName", app_name.lower().strip())
-        Tags.of(self).add("Environment", environment)
+        cdk.Tags.of(self).add("ApplicationName", app_name.lower().strip())
+        cdk.Tags.of(self).add("Environment", environment)
         
         # Apply FAO CDK tags
-        Tags.of(self).add("fao-cdk-construct", "service_user_for_iac")
-        Tags.of(core.Stack.of(self)).add("fao-cdk-version", get_version())
-        Tags.of(core.Stack.of(self)).add("fao-cdk", "true")
+        cdk.Tags.of(self).add("fao-cdk-construct", "service_user_for_iac")
+        cdk.Tags.of(cdk.Stack.of(self)).add("fao-cdk-version", get_version())
+        cdk.Tags.of(cdk.Stack.of(self)).add("fao-cdk", "true")
 
         # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Create conditions
 
@@ -85,9 +88,9 @@ class ServiceUserForIAC(core.Construct):
         # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Create AWS resources
 
         # Manage policy for IAC deployment
-        managed_policy_for_iac = _iam.ManagedPolicy(
+        managed_policy_for_iac_1 = _iam.ManagedPolicy(
             self,
-            "managed_policy",
+            "managed_policy_iac_one",
             description=app_name + " managed policy for IAC",
             statements=[
                 # Resource ALL
@@ -95,8 +98,10 @@ class ServiceUserForIAC(core.Construct):
                     actions=[
                         "cloudformation:Describe*",
                         "cloudformation:ValidateTemplate",
-                        "cloudformation:CreateChangeSet",
-                        "cloudformation:ExecuteChangeSe",
+                        "cloudformation:ExecuteChangeSet",
+                        "cloudformation:*ChangeSet*",
+                        "cloudformation:GetTemplate",
+                        "cloudformation:ListStackResources",
                         "ec2:CreateSecurityGroup",
                         "ec2:*SecurityGroup*",
                         "ec2:Describe*",
@@ -125,8 +130,7 @@ class ServiceUserForIAC(core.Construct):
                         "iam:AttachUserPolicy",
                         "iam:PassRole",
                         "iam:DeleteAccessKey",
-                        "rds:DescribeEngineDefaultParameters",
-                        "rds:DescribeEvents",
+                        "rds:Describe*",
                         "elasticfilesystem:*MountTarget*",
                         "elasticfilesystem:DescribeMountTargets",
                         "elasticfilesystem:DescribeFileSystems",
@@ -137,7 +141,15 @@ class ServiceUserForIAC(core.Construct):
                         "elasticfilesystem:UntagResource",
                         "autoscaling:DeletePolicy",
                         "cognito-idp:*UserPoolClient",
-                        "cloudfront:TagResource"
+                        "cloudfront:TagResource",
+                        "ecs:Describe*",
+                        "ecs:List*",
+                        "ecs:Create*",
+                        "ecs:DiscoverPollEndpoint",
+                        "ecs:TagResource",
+                        "iam:ListAttachedRolePolicies",
+                        "iam:ListInstanceProfiles",
+                        "iam:ListRoles",
                     ],
                     resources=["*"],
                 ),
@@ -274,6 +286,14 @@ class ServiceUserForIAC(core.Construct):
                         "arn:aws:cloudwatch:*:alarm:" + app_name + "*",
                     ],
                 ),
+            ]
+        )
+
+        managed_policy_for_iac_2 = _iam.ManagedPolicy(
+            self,
+            "managed_policy_iac_two",
+            description=app_name + " managed policy for IAC 2",
+            statements=[
                 # Filter by TAG
                 _iam.PolicyStatement(
                     actions=[
@@ -345,6 +365,104 @@ class ServiceUserForIAC(core.Construct):
                             }
                         },
                 ),
+                _iam.PolicyStatement(
+                    actions=[
+                        "ecs:*",
+                    ],
+                    resources=["*"],
+                    conditions={
+                            "StringLike": {
+                                "ecs:ResourceTag/ApplicationName": app_name
+                            }
+                        },
+                ),
+                _iam.PolicyStatement(
+                    actions=[
+                        "ecr:*",
+                    ],
+                    resources=["*"],
+                    conditions={
+                            "StringLike": {
+                                "ecr:ResourceTag/ApplicationName": app_name
+                            }
+                        },
+                ),
+                # Relative to ECS
+                _iam.PolicyStatement(
+                    actions=[
+                        "ssm:GetParameter",
+                        "ssm:GetParameters",
+                        "ssm:GetParametersByPath"
+                    ],
+                    resources=[
+                        "arn:aws:ssm:*:*:parameter/aws/service/ecs*"
+                    ],
+                ),
+	             _iam.PolicyStatement(
+                    actions=[
+                        "iam:PassRole"
+                        ],
+                    resources=[
+                        "*"
+                        ],
+                    conditions={"StringLike": {"iam:PassedToService": "ecs-tasks.amazonaws.com"}}
+                    ),
+                _iam.PolicyStatement(
+                    actions=[
+                        "iam:PassRole",
+                        ],
+                    resources=[
+                        "arn:aws:iam::*:role/ecsInstanceRole*"
+                        ],
+                    conditions={"StringLike": {"iam:PassedToService": [
+                            "ec2.amazonaws.com",
+                            "ec2.amazonaws.com.cn"
+                        ]}}
+                    ),
+                _iam.PolicyStatement(
+                    actions=[
+                        "iam:PassRole",
+                        ],
+                    resources=[
+                        "arn:aws:iam::*:role/ecsAutoscaleRole*"
+                        ],
+                    conditions={"StringLike": {"iam:PassedToService": [
+                            "application-autoscaling.amazonaws.com",
+                            "application-autoscaling.amazonaws.com.cn"
+                        ]}}
+                    ),
+                _iam.PolicyStatement(
+                    actions=[
+                        "iam:CreateServiceLinkedRole",
+                        ],
+                    resources=[
+                        "*"
+                        ],
+                    conditions={"StringLike": {"iam:AWSServiceName": [
+                            "autoscaling.amazonaws.com",
+                            "ecs.amazonaws.com",
+                            "ecs.application-autoscaling.amazonaws.com",
+                            "spot.amazonaws.com",
+                            "spotfleet.amazonaws.com"
+                        ]}}
+                    ),
+                _iam.PolicyStatement(
+                    actions=[
+                        "ecr:*"
+                        ],
+                    resources=[
+                        "arn:aws:ecr:eu-west-1:*:*" + app_name + "*"
+                        ],
+                    ),
+                _iam.PolicyStatement(
+                    actions=[
+                        "ecr:GetAuthorizationToken",
+                        "ecr:DescribeRepositories"
+                        ],
+                    resources=[
+                        "*"
+                        ],
+                    ),
                 # Filter by SPECIFIC RESOURCE
                 _iam.PolicyStatement(
                     actions=[
@@ -417,14 +535,50 @@ class ServiceUserForIAC(core.Construct):
                     ),
                 ],
             )
+
+        if not is_production:
+            # Route53 records
+            dns_statement = _iam.PolicyStatement(
+                actions=[
+                    "route53:ChangeResourceRecordSets",
+                    "route53:GetHostedZone*",
+                    "route53:ListHostedZones*",
+                    ],
+                resources=[
+                    "*"
+                    ],
+            )
+            managed_policy_for_iac_2.add_statements(dns_statement)
+           
+
+        # Managed policy for Configuration deployement
+        managed_policy_for_cdk =  _iam.ManagedPolicy(
+                self,
+                "service_user_policy_cdk",
+                statements=[
+                   _iam.PolicyStatement(
+                    actions=[
+                        "ssm:Describe*",
+                        "ssm:Get*",
+                        "ssm:List*",
+                    ],
+                    resources=[
+                        "arn:aws:ssm:*:*:*cdk*",
+                        "arn:aws:ssm:*:*:parameter/cdk*"
+                    ],
+                  )
+                ],
+            )
     
         # Service user
         self._service_user = _iam.User(
             self,
             "service_user",
             managed_policies=[
-                managed_policy_for_iac,
+                managed_policy_for_iac_1,
+                managed_policy_for_iac_2,
                 managed_policy_for_static_assets,
+                managed_policy_for_cdk
                 ],
             user_name="SRVUSR-" + app_name + "_" + environment + "_iac",
         )
